type token =
  | FLOAT of (string)
  | INT of (NumConst.t)
  | NAME of (string)
  | STR_FLOAT of (string)
  | STR_INT of (NumConst.t)
  | OP_MINUS
  | OP_DIV
  | OP_EQ
  | OPEN_ARC
  | OPEN_ATTRIBUTE
  | OPEN_NODE
  | OPEN_XML
  | OPEN_END_ARC
  | OPEN_END_ATTRIBUTE
  | OPEN_END_MODEL
  | OPEN_END_NODE
  | CLOSE
  | CLOSE_XML
  | SINGLE_CLOSE
  | INITIAL
  | NORMAL
  | FINAL
  | TRUE
  | XML_HEADER
  | MODEL_HEADER
  | STR_AND
  | STR_BOOLEXPR
  | STR_BOOLVALUE
  | STR_CLOCK
  | STR_CLOCKS
  | STR_CONST
  | STR_CONSTANTS
  | STR_DECLARATION
  | STR_GLOBALCONSTANTS
  | STR_DISCRETE
  | STR_DISCRETES
  | STR_EXPR
  | STR_GUARD
  | STR_INITIALCONSTRAINT
  | STR_INVARIANT
  | STR_LABEL
  | STR_NAME
  | STR_PARAMETER
  | STR_PARAMETERS
  | STR_STATE
  | STR_TRANSITION
  | STR_TYPE
  | STR_UPDATE
  | STR_UPDATES
  | STR_VARIABLES
  | STR_OPL
  | STR_OPLEQ
  | STR_OPEQ
  | STR_OPGEQ
  | STR_OPG
  | STR_OPMUL
  | CT_ARCTYPE
  | CT_ID
  | CT_NAME
  | CT_NODETYPE
  | CT_SOURCE
  | CT_TARGET
  | EOF

open Parsing;;
# 17 "src/GMLParser.mly"
open ParsingStructure;;
open Global;;
open NumConst;;

(* Counter for the unnamed locations *)
let nb_unnamed_locations = ref 0;;

(* Counter for the names of the automata*)
let automata_names = ref 0;;

let parse_error s =
	let symbol_start = symbol_start () in
	let symbol_end = symbol_end () in
	raise (ParsingError (symbol_start, symbol_end))
;;

let  merge_declarations local_declarations declarations  =
     List.append local_declarations declarations
;;

let  merge_automata local_automaton automata =
     List.append local_automaton automata
;;   


let merge_init_definitions local_init_definitions init_definitions =
    List.append local_init_definitions init_definitions
;; 

let convert declarations locations transitions =
	(* SO FAR CONSIDER ONLY ONE AUTOMATON *)
	let init_location : string ref = ref "" in
	let ok = ref true in
	
	(* Hashtbl location_id -> location invariant *)
	let invariants = Hashtbl.create (List.length locations) in
	(* Hashtbl location_id -> name *)
	let location_names : (NumConst.t, string) Hashtbl.t = Hashtbl.create (List.length locations) in
	(* Hashtbl location_id -> list of transtions *)
	let location_transitions = Hashtbl.create (List.length locations) in
	(* List of sync labs *)
	let synclabs = ref [] in
	
	(* Iterate on location descriptions *)
	List.iter (fun location -> 
		let (location_id, (location_name, (is_init, is_final), invariant)) = location in
			(* Check if id is defined *)
			if (Hashtbl.mem location_names location_id) then (
				print_error ("Location id '" ^ (string_of_numconst location_id) ^ "' is used several times.");
				ok := false;
			);
			(* Warn if final *)
			if is_final then print_warning ("Location '" ^ location_name ^ "' is defined as final; this will not be taken into account.");
			(* Check if initial *)
			if is_init then (
				(* Check if another initial state is defined *)
				if !init_location <> "" then (
					print_error ("At least two locations ('" ^ location_name ^ "' and '" ^ (!init_location) ^ "') are defined as initial.");
					ok := false;
				);
				(* Update init *)
				init_location := location_name;
			);
			(* Add couple id, name *)
			Hashtbl.add location_names location_id location_name;
			(* Add couple id, invariant *)
			Hashtbl.add invariants location_id invariant;
	) locations;
	
	(* Iterate on transitions descriptions *)
	List.iter (fun transition -> 
		let source_id, target_id, (label, guard, updates) = transition in
			(* Find source and target names *)
			if Hashtbl.mem location_names source_id && Hashtbl.mem location_names target_id then (
(* 				let source_name = Hashtbl.find location_names source_id in *)
				let target_name = Hashtbl.find location_names target_id in
				(* type transition = guard * update list * sync * location_name *)
				let new_transition = guard, updates, label, target_name in
				(* Add transition *)
				let _ =
				try (
					let list_of_transitions = Hashtbl.find location_transitions source_id in
					Hashtbl.replace location_transitions source_id (new_transition :: list_of_transitions);
				) with Not_found -> (
					Hashtbl.add location_transitions source_id [new_transition];
				);
				in ();
				(* Add sync lab if real sync lab *)
				match label with
					| NoSync -> ()
					| Sync sync_name -> synclabs := sync_name :: !synclabs;
			) else (
				ok := false;
				if not (Hashtbl.mem location_names source_id) then
					print_error ("No source location of id '" ^ (string_of_numconst source_id) ^ "' has been defined.");
				if not (Hashtbl.mem location_names target_id) then
					print_error ("No target location of id '" ^ (string_of_numconst target_id) ^ "' has been defined.");
			);
	) transitions;
	
	(* Don't go further *)
	if (not !ok) then raise InvalidModel;

	(* Make transitions *)
	let locations = Hashtbl.fold (fun location_id location_name current_list ->
		(* Get info *)
		let location_name = Hashtbl.find location_names location_id in
		let transitions = try (Hashtbl.find location_transitions location_id) with Not_found -> [] in
		let invariant = Hashtbl.find invariants location_id in
		(* Make structure *)
		(* WARNING: stopwatches and costs not taken into account yet !! *)
		let location = location_name , None , invariant , [], List.rev transitions in
		(* Add it *)
		location :: current_list
	) location_names [] in
		
	(* Allows different names for the automata *)
	automata_names := !automata_names + 1 ;
	let automaton_name = string_of_int(!automata_names) in 

	(* Name x list of synclabs x locations *)
	let automaton = automaton_name, list_only_once(!synclabs), List.rev locations in
	
	(* Make init *)
	
	let init_definition = 
		if (!init_location = "") then (
			[]
		)
		else [Loc_assignment (automaton_name , (!init_location))]
	in
	
	[automaton], init_definition
 
# 203 "src/GMLParser.ml"
let yytransl_const = [|
  262 (* OP_MINUS *);
  263 (* OP_DIV *);
  264 (* OP_EQ *);
  265 (* OPEN_ARC *);
  266 (* OPEN_ATTRIBUTE *);
  267 (* OPEN_NODE *);
  268 (* OPEN_XML *);
  269 (* OPEN_END_ARC *);
  270 (* OPEN_END_ATTRIBUTE *);
  271 (* OPEN_END_MODEL *);
  272 (* OPEN_END_NODE *);
  273 (* CLOSE *);
  274 (* CLOSE_XML *);
  275 (* SINGLE_CLOSE *);
  276 (* INITIAL *);
  277 (* NORMAL *);
  278 (* FINAL *);
  279 (* TRUE *);
  280 (* XML_HEADER *);
  281 (* MODEL_HEADER *);
  282 (* STR_AND *);
  283 (* STR_BOOLEXPR *);
  284 (* STR_BOOLVALUE *);
  285 (* STR_CLOCK *);
  286 (* STR_CLOCKS *);
  287 (* STR_CONST *);
  288 (* STR_CONSTANTS *);
  289 (* STR_DECLARATION *);
  290 (* STR_GLOBALCONSTANTS *);
  291 (* STR_DISCRETE *);
  292 (* STR_DISCRETES *);
  293 (* STR_EXPR *);
  294 (* STR_GUARD *);
  295 (* STR_INITIALCONSTRAINT *);
  296 (* STR_INVARIANT *);
  297 (* STR_LABEL *);
  298 (* STR_NAME *);
  299 (* STR_PARAMETER *);
  300 (* STR_PARAMETERS *);
  301 (* STR_STATE *);
  302 (* STR_TRANSITION *);
  303 (* STR_TYPE *);
  304 (* STR_UPDATE *);
  305 (* STR_UPDATES *);
  306 (* STR_VARIABLES *);
  307 (* STR_OPL *);
  308 (* STR_OPLEQ *);
  309 (* STR_OPEQ *);
  310 (* STR_OPGEQ *);
  311 (* STR_OPG *);
  312 (* STR_OPMUL *);
  313 (* CT_ARCTYPE *);
  314 (* CT_ID *);
  315 (* CT_NAME *);
  316 (* CT_NODETYPE *);
  317 (* CT_SOURCE *);
  318 (* CT_TARGET *);
    0 (* EOF *);
    0|]

let yytransl_block = [|
  257 (* FLOAT *);
  258 (* INT *);
  259 (* NAME *);
  260 (* STR_FLOAT *);
  261 (* STR_INT *);
    0|]

let yylhs = "\255\255\
\001\000\002\000\002\000\003\000\004\000\006\000\007\000\008\000\
\005\000\009\000\012\000\012\000\015\000\015\000\015\000\015\000\
\015\000\013\000\013\000\018\000\018\000\016\000\020\000\020\000\
\021\000\017\000\023\000\023\000\024\000\019\000\025\000\025\000\
\026\000\014\000\014\000\027\000\010\000\010\000\029\000\029\000\
\031\000\031\000\030\000\030\000\034\000\034\000\034\000\034\000\
\034\000\034\000\035\000\036\000\036\000\037\000\032\000\032\000\
\032\000\011\000\011\000\038\000\038\000\039\000\039\000\040\000\
\040\000\041\000\041\000\041\000\041\000\041\000\041\000\041\000\
\041\000\041\000\043\000\043\000\043\000\042\000\042\000\045\000\
\045\000\046\000\028\000\048\000\048\000\048\000\049\000\050\000\
\051\000\052\000\052\000\052\000\052\000\052\000\047\000\053\000\
\054\000\054\000\056\000\056\000\055\000\033\000\058\000\058\000\
\044\000\059\000\059\000\022\000\057\000\057\000\057\000\060\000\
\060\000\062\000\061\000\061\000\063\000\000\000"

let yylen = "\002\000\
\002\000\001\000\002\000\003\000\002\000\002\000\003\000\002\000\
\003\000\007\000\000\000\005\000\000\000\001\000\001\000\002\000\
\002\000\000\000\005\000\000\000\001\000\005\000\000\000\002\000\
\005\000\005\000\000\000\002\000\005\000\005\000\000\000\002\000\
\005\000\000\000\001\000\005\000\002\000\000\000\013\000\013\000\
\003\000\002\000\006\000\000\000\003\000\002\000\003\000\003\000\
\004\000\004\000\003\000\001\000\001\000\001\000\005\000\004\000\
\003\000\002\000\000\000\016\000\016\000\003\000\000\000\003\000\
\000\000\003\000\002\000\002\000\002\000\002\000\002\000\001\000\
\001\000\001\000\005\000\004\000\003\000\005\000\003\000\002\000\
\000\000\006\000\005\000\001\000\001\000\001\000\005\000\006\000\
\006\000\001\000\001\000\001\000\001\000\001\000\005\000\001\000\
\001\000\001\000\001\000\006\000\005\000\005\000\001\000\000\000\
\005\000\001\000\000\000\005\000\001\000\001\000\003\000\001\000\
\002\000\001\000\001\000\002\000\001\000\002\000"

let yydefred = "\000\000\
\000\000\000\000\000\000\118\000\000\000\002\000\000\000\005\000\
\001\000\003\000\000\000\000\000\000\000\000\000\000\000\000\000\
\004\000\000\000\000\000\000\000\000\000\007\000\006\000\000\000\
\000\000\000\000\000\000\009\000\000\000\037\000\000\000\000\000\
\000\000\000\000\000\000\000\000\058\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\035\000\000\000\000\000\000\000\062\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\010\000\000\000\000\000\000\000\064\000\000\000\000\000\000\000\
\000\000\012\000\000\000\016\000\000\000\017\000\000\000\000\000\
\021\000\000\000\008\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\019\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\036\000\000\000\000\000\000\000\000\000\
\000\000\000\000\022\000\024\000\000\000\026\000\028\000\000\000\
\000\000\000\000\000\000\043\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\030\000\032\000\000\000\000\000\084\000\
\085\000\086\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\090\000\
\091\000\092\000\093\000\094\000\000\000\083\000\000\000\000\000\
\000\000\000\000\000\000\000\000\042\000\000\000\000\000\000\000\
\000\000\000\000\025\000\029\000\000\000\000\000\000\000\000\000\
\000\000\057\000\000\000\039\000\041\000\000\000\052\000\053\000\
\054\000\046\000\000\000\000\000\040\000\000\000\000\000\000\000\
\033\000\000\000\000\000\000\000\000\000\056\000\000\000\103\000\
\000\000\051\000\045\000\047\000\000\000\048\000\000\000\000\000\
\000\000\000\000\000\000\087\000\000\000\000\000\055\000\102\000\
\049\000\050\000\000\000\000\000\000\000\000\000\000\000\000\000\
\108\000\088\000\000\000\089\000\000\000\000\000\000\000\000\000\
\000\000\000\000\068\000\000\000\071\000\000\000\070\000\069\000\
\000\000\000\000\099\000\000\000\096\000\097\000\098\000\000\000\
\077\000\000\000\000\000\079\000\060\000\000\000\066\000\061\000\
\000\000\000\000\095\000\076\000\000\000\106\000\000\000\000\000\
\000\000\000\000\000\000\000\000\075\000\105\000\000\000\078\000\
\080\000\117\000\114\000\000\000\000\000\000\000\110\000\112\000\
\115\000\000\000\000\000\000\000\113\000\116\000\101\000\000\000\
\000\000\000\000\111\000\100\000\000\000\082\000"

let yydgoto = "\002\000\
\004\000\005\000\006\000\007\000\012\000\017\000\093\000\065\000\
\014\000\020\000\028\000\032\000\040\000\049\000\059\000\060\000\
\061\000\080\000\081\000\101\000\102\000\147\000\104\000\105\000\
\121\000\122\000\050\000\094\000\021\000\052\000\140\000\141\000\
\142\000\165\000\166\000\187\000\188\000\029\000\036\000\045\000\
\220\000\221\000\222\000\223\000\009\001\010\001\197\000\135\000\
\136\000\137\000\138\000\157\000\244\000\245\000\246\000\247\000\
\021\001\201\000\007\001\022\001\023\001\024\001\025\001"

let yysindex = "\068\000\
\061\255\000\000\064\255\000\000\001\000\000\000\096\255\000\000\
\000\000\000\000\034\255\098\255\081\255\109\255\116\255\108\255\
\000\000\110\255\017\255\117\255\109\255\000\000\000\000\096\255\
\120\255\121\255\072\255\000\000\117\255\000\000\086\255\096\255\
\128\255\093\255\129\255\087\255\000\000\123\255\114\255\096\255\
\138\255\138\255\142\255\144\255\226\254\096\255\131\255\111\255\
\139\255\000\000\149\255\099\255\104\255\000\000\118\255\155\255\
\157\255\014\255\139\255\096\255\096\255\096\255\150\255\151\255\
\000\000\161\255\162\255\163\255\000\000\167\255\168\255\158\255\
\159\255\000\000\141\255\000\000\148\255\000\000\130\255\139\255\
\000\000\096\255\000\000\176\255\135\255\177\255\087\255\087\255\
\096\255\096\255\164\255\000\000\156\255\139\255\178\255\138\255\
\138\255\122\255\124\255\160\255\139\255\096\255\152\255\139\255\
\096\255\096\255\179\255\000\000\186\255\180\255\181\255\185\255\
\187\255\182\255\000\000\000\000\183\255\000\000\000\000\165\255\
\139\255\096\255\096\255\000\000\096\255\096\255\189\255\198\255\
\096\255\096\255\193\255\000\000\000\000\008\255\139\255\000\000\
\000\000\000\000\036\255\188\255\096\255\096\255\190\255\072\255\
\072\255\169\255\139\255\139\255\096\255\195\255\196\255\000\000\
\000\000\000\000\000\000\000\000\201\255\000\000\071\255\202\255\
\205\255\175\255\096\255\184\255\000\000\021\255\206\255\087\255\
\087\255\207\255\000\000\000\000\139\255\096\255\203\255\096\255\
\060\255\000\000\222\255\000\000\000\000\077\255\000\000\000\000\
\000\000\000\000\006\255\026\255\000\000\211\255\212\255\227\255\
\000\000\096\255\139\255\197\255\096\255\000\000\139\255\000\000\
\139\255\000\000\000\000\000\000\139\255\000\000\139\255\096\255\
\096\255\139\255\139\255\000\000\215\255\139\255\000\000\000\000\
\000\000\000\000\229\254\223\255\096\255\096\255\096\255\225\255\
\000\000\000\000\096\255\000\000\083\255\218\255\084\255\224\255\
\234\254\096\255\000\000\191\255\000\000\236\254\000\000\000\000\
\226\255\237\254\000\000\139\255\000\000\000\000\000\000\060\255\
\000\000\236\255\096\255\000\000\000\000\204\255\000\000\000\000\
\229\255\230\255\000\000\000\000\139\255\000\000\139\255\194\255\
\139\255\096\255\065\255\096\255\000\000\000\000\231\255\000\000\
\000\000\000\000\000\000\050\255\139\255\242\255\000\000\000\000\
\000\000\219\255\096\255\096\255\000\000\000\000\000\000\250\255\
\139\255\096\255\000\000\000\000\139\255\000\000"

let yyrindex = "\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\049\255\000\000\000\000\
\000\000\000\000\000\000\238\255\049\255\000\000\000\000\237\255\
\000\000\000\000\011\255\000\000\238\255\000\000\000\000\240\255\
\000\000\000\000\000\000\019\255\000\000\000\000\000\000\241\255\
\208\255\199\255\000\000\000\000\000\000\244\255\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\246\255\248\255\251\255\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\209\255\213\255\
\252\255\253\255\000\000\000\000\000\000\000\000\000\000\239\255\
\239\255\000\000\000\000\000\000\000\000\252\255\000\000\000\000\
\253\255\255\255\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\255\255\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\248\254\
\248\254\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\247\255\
\247\255\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\002\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\004\000\006\000\009\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\010\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\012\000\014\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\014\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\015\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000"

let yygindex = "\000\000\
\000\000\000\000\002\001\000\000\000\000\000\000\249\255\200\255\
\000\000\249\000\251\000\000\000\000\000\000\000\000\000\225\000\
\227\000\000\000\000\000\186\000\000\000\133\255\184\000\000\000\
\168\000\000\000\000\000\092\255\000\000\216\255\165\000\000\000\
\151\000\130\000\000\000\106\000\108\000\000\000\220\255\174\255\
\087\000\145\255\140\255\043\255\031\000\000\000\063\255\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\030\000\000\000\
\000\000\000\000\000\000\000\000\000\000\251\254\023\000"

let yytablesize = 299
let yytable = "\013\000\
\009\000\053\000\074\000\214\000\098\000\099\000\148\000\235\000\
\063\000\194\000\229\000\001\001\199\000\230\000\029\001\229\000\
\031\000\229\000\230\000\064\000\255\000\231\000\170\000\092\000\
\039\000\173\000\035\001\185\000\231\000\211\000\056\000\057\000\
\048\000\150\000\064\000\151\000\002\001\108\000\058\000\064\000\
\183\000\184\000\185\000\072\000\115\000\183\000\184\000\118\000\
\063\000\073\000\018\001\019\001\075\000\077\000\079\000\110\000\
\111\000\038\000\152\000\153\000\154\000\155\000\156\000\038\000\
\132\000\018\001\019\001\063\000\001\000\011\000\020\001\063\000\
\063\000\064\000\025\000\159\000\026\000\160\000\158\000\065\000\
\065\000\100\000\103\000\005\001\003\000\190\000\191\000\177\000\
\008\000\178\000\171\000\172\000\015\000\202\000\100\000\203\000\
\037\001\103\000\120\000\248\000\251\000\249\000\252\000\243\000\
\234\000\011\000\240\000\168\000\169\000\186\000\237\000\239\000\
\016\000\018\000\120\000\134\000\193\000\139\000\139\000\019\000\
\198\000\146\000\146\000\022\000\023\000\027\000\024\000\033\000\
\034\000\035\000\204\000\206\000\041\000\162\000\164\000\038\000\
\043\000\042\000\212\000\046\000\051\000\146\000\215\000\044\000\
\216\000\047\000\054\000\062\000\217\000\063\000\218\000\055\000\
\064\000\225\000\226\000\164\000\066\000\228\000\067\000\033\001\
\034\001\068\000\070\000\069\000\071\000\084\000\082\000\083\000\
\196\000\085\000\086\000\087\000\088\000\091\000\089\000\090\000\
\073\000\072\000\095\000\096\000\106\000\097\000\107\000\112\000\
\113\000\109\000\117\000\003\001\114\000\196\000\124\000\004\001\
\127\000\144\000\128\000\123\000\125\000\126\000\129\000\130\000\
\219\000\219\000\145\000\161\000\013\001\167\000\014\001\131\000\
\016\001\149\000\170\000\174\000\175\000\233\000\236\000\238\000\
\160\000\176\000\179\000\242\000\031\001\180\000\189\000\192\000\
\200\000\195\000\254\000\208\000\209\000\210\000\182\000\227\000\
\036\001\213\000\250\000\232\000\038\001\241\000\006\001\231\000\
\253\000\015\001\000\001\008\001\230\000\011\001\012\001\028\001\
\032\001\001\001\011\000\019\001\059\000\018\000\034\000\044\000\
\044\000\013\000\008\001\014\000\026\001\015\000\010\000\065\000\
\020\000\023\000\027\000\044\000\031\000\030\000\065\000\104\000\
\074\000\065\000\073\000\146\000\146\000\072\000\067\000\037\000\
\003\000\107\000\196\000\081\000\109\000\078\000\076\000\116\000\
\119\000\133\000\143\000\163\000\181\000\207\000\205\000\224\000\
\017\001\027\001\030\001"

let yycheck = "\007\000\
\000\000\042\000\059\000\197\000\087\000\088\000\130\000\221\000\
\017\001\174\000\038\001\031\001\177\000\041\001\020\001\038\001\
\024\000\038\001\041\001\014\001\234\000\049\001\042\001\080\000\
\032\000\149\000\032\001\022\001\049\001\194\000\061\001\062\001\
\040\000\026\001\014\001\028\001\056\001\094\000\046\000\014\001\
\020\001\021\001\022\001\030\001\101\000\020\001\021\001\104\000\
\057\001\036\001\001\001\002\001\060\000\061\000\062\000\096\000\
\097\000\009\001\051\001\052\001\053\001\054\001\055\001\015\001\
\121\000\001\001\002\001\057\001\001\000\010\001\006\001\061\001\
\062\001\014\001\058\001\040\001\060\001\042\001\135\000\061\001\
\062\001\089\000\090\000\248\000\024\001\168\000\169\000\017\001\
\025\001\019\001\147\000\148\000\059\001\017\001\102\000\019\001\
\034\001\105\000\106\000\017\001\017\001\019\001\019\001\227\000\
\221\000\010\001\223\000\144\000\145\000\166\000\222\000\223\000\
\015\001\033\001\122\000\123\000\173\000\125\000\126\000\011\001\
\177\000\129\000\130\000\008\001\017\001\009\001\017\001\008\001\
\008\001\058\001\187\000\188\000\005\001\141\000\142\000\050\001\
\008\001\045\001\195\000\017\001\003\001\149\000\199\000\057\001\
\201\000\032\001\005\001\017\001\205\000\039\001\207\000\008\001\
\014\001\210\000\211\000\163\000\008\001\214\000\060\001\027\001\
\028\001\058\001\008\001\046\001\008\001\005\001\017\001\017\001\
\176\000\008\001\008\001\005\001\005\001\044\001\017\001\017\001\
\036\001\030\001\003\001\045\001\017\001\005\001\027\001\062\001\
\061\001\008\001\035\001\244\000\029\001\197\000\005\001\248\000\
\008\001\005\001\008\001\017\001\017\001\017\001\017\001\017\001\
\208\000\209\000\005\001\016\001\005\001\016\001\007\001\043\001\
\009\001\017\001\042\001\017\001\017\001\221\000\222\000\223\000\
\042\001\017\001\017\001\227\000\021\001\017\001\017\001\017\001\
\003\001\023\001\234\000\017\001\017\001\003\001\047\001\017\001\
\033\001\037\001\017\001\013\001\037\001\013\001\003\001\049\001\
\017\001\048\001\017\001\251\000\041\001\017\001\017\001\017\001\
\007\001\031\001\014\001\002\001\015\001\014\001\014\001\017\001\
\058\001\014\001\010\001\014\001\012\001\014\001\005\000\017\001\
\014\001\014\001\014\001\060\001\014\001\021\000\062\001\014\001\
\013\001\061\001\013\001\027\001\028\001\013\001\013\001\029\000\
\024\001\014\001\034\001\014\001\014\001\061\000\060\000\102\000\
\105\000\122\000\126\000\141\000\163\000\188\000\187\000\209\000\
\010\001\012\001\020\001"

let yynames_const = "\
  OP_MINUS\000\
  OP_DIV\000\
  OP_EQ\000\
  OPEN_ARC\000\
  OPEN_ATTRIBUTE\000\
  OPEN_NODE\000\
  OPEN_XML\000\
  OPEN_END_ARC\000\
  OPEN_END_ATTRIBUTE\000\
  OPEN_END_MODEL\000\
  OPEN_END_NODE\000\
  CLOSE\000\
  CLOSE_XML\000\
  SINGLE_CLOSE\000\
  INITIAL\000\
  NORMAL\000\
  FINAL\000\
  TRUE\000\
  XML_HEADER\000\
  MODEL_HEADER\000\
  STR_AND\000\
  STR_BOOLEXPR\000\
  STR_BOOLVALUE\000\
  STR_CLOCK\000\
  STR_CLOCKS\000\
  STR_CONST\000\
  STR_CONSTANTS\000\
  STR_DECLARATION\000\
  STR_GLOBALCONSTANTS\000\
  STR_DISCRETE\000\
  STR_DISCRETES\000\
  STR_EXPR\000\
  STR_GUARD\000\
  STR_INITIALCONSTRAINT\000\
  STR_INVARIANT\000\
  STR_LABEL\000\
  STR_NAME\000\
  STR_PARAMETER\000\
  STR_PARAMETERS\000\
  STR_STATE\000\
  STR_TRANSITION\000\
  STR_TYPE\000\
  STR_UPDATE\000\
  STR_UPDATES\000\
  STR_VARIABLES\000\
  STR_OPL\000\
  STR_OPLEQ\000\
  STR_OPEQ\000\
  STR_OPGEQ\000\
  STR_OPG\000\
  STR_OPMUL\000\
  CT_ARCTYPE\000\
  CT_ID\000\
  CT_NAME\000\
  CT_NODETYPE\000\
  CT_SOURCE\000\
  CT_TARGET\000\
  EOF\000\
  "

let yynames_block = "\
  FLOAT\000\
  INT\000\
  NAME\000\
  STR_FLOAT\000\
  STR_INT\000\
  "

let yyact = [|
  (fun _ -> failwith "parser")
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'models) in
    Obj.repr(
# 192 "src/GMLParser.mly"
                  (_1)
# 601 "src/GMLParser.ml"
               : ParsingStructure.parsing_structure))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'model) in
    Obj.repr(
# 195 "src/GMLParser.mly"
               (_1)
# 608 "src/GMLParser.ml"
               : 'models))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'models) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'model) in
    Obj.repr(
# 196 "src/GMLParser.mly"
       (      
	 let declarations, automata, init_definitions, _, _  = _1 in
	 let local_declarations, local_automaton, local_init_definitions, _ , _ = _2 in
	 let all_declarations = merge_declarations local_declarations declarations  in
	 let all_automata = merge_automata local_automaton automata in
	 let all_init_definitions = merge_init_definitions local_init_definitions init_definitions in 
			(*** HACK: need to consider bad and carto as well (or?) *)
	  		  all_declarations, all_automata, all_init_definitions, None, ([] , (NumConst.zero,NumConst.zero) , (NumConst.zero,NumConst.zero))
	)
# 624 "src/GMLParser.ml"
               : 'models))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'header) in
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'body) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'footer) in
    Obj.repr(
# 213 "src/GMLParser.mly"
 (
		let declarations, states, transitions = _2 in
		let automata, init_definition = convert declarations states transitions in
		(*** HACK: need to consider bad and carto as well (or?) *)
		declarations, automata, init_definition, None, ([] , (NumConst.zero,NumConst.zero) , (NumConst.zero,NumConst.zero))
	)
# 638 "src/GMLParser.ml"
               : 'model))
; (fun __caml_parser_env ->
    Obj.repr(
# 232 "src/GMLParser.mly"
 ()
# 644 "src/GMLParser.ml"
               : 'header))
; (fun __caml_parser_env ->
    Obj.repr(
# 237 "src/GMLParser.mly"
                      ( )
# 650 "src/GMLParser.ml"
               : 'footer))
; (fun __caml_parser_env ->
    Obj.repr(
# 244 "src/GMLParser.mly"
                              ( )
# 656 "src/GMLParser.ml"
               : 'open_attribute))
; (fun __caml_parser_env ->
    Obj.repr(
# 248 "src/GMLParser.mly"
                          ( )
# 662 "src/GMLParser.ml"
               : 'close_attribute))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'declarations) in
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'states) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'transitions) in
    Obj.repr(
# 258 "src/GMLParser.mly"
                                   ( _1, _2, _3 )
# 671 "src/GMLParser.ml"
               : 'body))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 6 : 'open_attribute) in
    let _4 = (Parsing.peek_val __caml_parser_env 3 : 'variables) in
    let _5 = (Parsing.peek_val __caml_parser_env 2 : 'constants) in
    let _6 = (Parsing.peek_val __caml_parser_env 1 : 'initial) in
    let _7 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 270 "src/GMLParser.mly"
  ( List.rev_append _4 _5 )
# 682 "src/GMLParser.ml"
               : 'declarations))
; (fun __caml_parser_env ->
    Obj.repr(
# 280 "src/GMLParser.mly"
   ( [] )
# 688 "src/GMLParser.ml"
               : 'variables))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : 'open_attribute) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'variables_body) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 284 "src/GMLParser.mly"
                  ( _4 )
# 697 "src/GMLParser.ml"
               : 'variables))
; (fun __caml_parser_env ->
    Obj.repr(
# 289 "src/GMLParser.mly"
   ( [] )
# 703 "src/GMLParser.ml"
               : 'variables_body))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'clocks) in
    Obj.repr(
# 290 "src/GMLParser.mly"
          ( [_1] )
# 710 "src/GMLParser.ml"
               : 'variables_body))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'discretes) in
    Obj.repr(
# 291 "src/GMLParser.mly"
             ( [_1] )
# 717 "src/GMLParser.ml"
               : 'variables_body))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'clocks) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'discretes) in
    Obj.repr(
# 292 "src/GMLParser.mly"
                    ( [ _1 ; _2 ] )
# 725 "src/GMLParser.ml"
               : 'variables_body))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'discretes) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'clocks) in
    Obj.repr(
# 293 "src/GMLParser.mly"
                    ( [ _1 ; _2 ] )
# 733 "src/GMLParser.ml"
               : 'variables_body))
; (fun __caml_parser_env ->
    Obj.repr(
# 303 "src/GMLParser.mly"
   ( [] )
# 739 "src/GMLParser.ml"
               : 'constants))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : 'open_attribute) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'constants_body) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 307 "src/GMLParser.mly"
                  ( _4 )
# 748 "src/GMLParser.ml"
               : 'constants))
; (fun __caml_parser_env ->
    Obj.repr(
# 312 "src/GMLParser.mly"
   ( [] )
# 754 "src/GMLParser.ml"
               : 'constants_body))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'parameters) in
    Obj.repr(
# 315 "src/GMLParser.mly"
              ( [_1] )
# 761 "src/GMLParser.ml"
               : 'constants_body))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : 'open_attribute) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'list_of_clocks) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 329 "src/GMLParser.mly"
                  ( (Var_type_clock, _4) )
# 770 "src/GMLParser.ml"
               : 'clocks))
; (fun __caml_parser_env ->
    Obj.repr(
# 333 "src/GMLParser.mly"
   ( [] )
# 776 "src/GMLParser.ml"
               : 'list_of_clocks))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'clock) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'list_of_clocks) in
    Obj.repr(
# 334 "src/GMLParser.mly"
                        ( _1 :: _2 )
# 784 "src/GMLParser.ml"
               : 'list_of_clocks))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : 'open_attribute) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'name) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 343 "src/GMLParser.mly"
   ( (_4, None) )
# 793 "src/GMLParser.ml"
               : 'clock))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : 'open_attribute) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'list_of_discretes) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 353 "src/GMLParser.mly"
                  ( (Var_type_discrete, _4) )
# 802 "src/GMLParser.ml"
               : 'discretes))
; (fun __caml_parser_env ->
    Obj.repr(
# 357 "src/GMLParser.mly"
   ( [] )
# 808 "src/GMLParser.ml"
               : 'list_of_discretes))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'discrete) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'list_of_discretes) in
    Obj.repr(
# 358 "src/GMLParser.mly"
                              ( _1 :: _2 )
# 816 "src/GMLParser.ml"
               : 'list_of_discretes))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : 'open_attribute) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'name) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 367 "src/GMLParser.mly"
   ( (_4, None) )
# 825 "src/GMLParser.ml"
               : 'discrete))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : 'open_attribute) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'list_of_parameters) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 390 "src/GMLParser.mly"
                  ( (Var_type_parameter, _4) )
# 834 "src/GMLParser.ml"
               : 'parameters))
; (fun __caml_parser_env ->
    Obj.repr(
# 394 "src/GMLParser.mly"
   ( [] )
# 840 "src/GMLParser.ml"
               : 'list_of_parameters))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'parameter) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'list_of_parameters) in
    Obj.repr(
# 395 "src/GMLParser.mly"
                                ( _1 :: _2 )
# 848 "src/GMLParser.ml"
               : 'list_of_parameters))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : 'open_attribute) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'name) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 404 "src/GMLParser.mly"
   ( (_4, None) )
# 857 "src/GMLParser.ml"
               : 'parameter))
; (fun __caml_parser_env ->
    Obj.repr(
# 411 "src/GMLParser.mly"
   ( [] )
# 863 "src/GMLParser.ml"
               : 'initial))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'initial_constraint) in
    Obj.repr(
# 412 "src/GMLParser.mly"
                      ( _1 )
# 870 "src/GMLParser.ml"
               : 'initial))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : 'open_attribute) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'bool_expr) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 418 "src/GMLParser.mly"
                 ( _4 )
# 879 "src/GMLParser.ml"
               : 'initial_constraint))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'state) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'states) in
    Obj.repr(
# 426 "src/GMLParser.mly"
                ( _1 :: _2 )
# 887 "src/GMLParser.ml"
               : 'states))
; (fun __caml_parser_env ->
    Obj.repr(
# 427 "src/GMLParser.mly"
   ( [] )
# 893 "src/GMLParser.ml"
               : 'states))
; (fun __caml_parser_env ->
    let _4 = (Parsing.peek_val __caml_parser_env 9 : NumConst.t) in
    let _5 = (Parsing.peek_val __caml_parser_env 8 : 'coordinates_opt) in
    let _9 = (Parsing.peek_val __caml_parser_env 4 : 'coordinates_opt) in
    let _11 = (Parsing.peek_val __caml_parser_env 2 : 'state_attributes) in
    Obj.repr(
# 437 "src/GMLParser.mly"
                      ( _4, _11 )
# 903 "src/GMLParser.ml"
               : 'state))
; (fun __caml_parser_env ->
    let _5 = (Parsing.peek_val __caml_parser_env 8 : 'coordinates_opt) in
    let _8 = (Parsing.peek_val __caml_parser_env 5 : NumConst.t) in
    let _9 = (Parsing.peek_val __caml_parser_env 4 : 'coordinates_opt) in
    let _11 = (Parsing.peek_val __caml_parser_env 2 : 'state_attributes) in
    Obj.repr(
# 440 "src/GMLParser.mly"
                      ( _8, _11 )
# 913 "src/GMLParser.ml"
               : 'state))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'state_invariant) in
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'state_name) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'state_type) in
    Obj.repr(
# 448 "src/GMLParser.mly"
                                         ( _2, _3, _1 )
# 922 "src/GMLParser.ml"
               : 'state_attributes))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'state_name) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'state_type) in
    Obj.repr(
# 449 "src/GMLParser.mly"
                         ( _1, _2, [] )
# 930 "src/GMLParser.ml"
               : 'state_attributes))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 5 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 3 : NumConst.t) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : string) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : NumConst.t) in
    Obj.repr(
# 467 "src/GMLParser.mly"
                                         ( )
# 940 "src/GMLParser.ml"
               : 'coordinates_opt))
; (fun __caml_parser_env ->
    Obj.repr(
# 468 "src/GMLParser.mly"
   ( )
# 946 "src/GMLParser.ml"
               : 'coordinates_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'open_attribute) in
    Obj.repr(
# 478 "src/GMLParser.mly"
                                        ( false, false )
# 953 "src/GMLParser.ml"
               : 'state_type))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'state_open) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 482 "src/GMLParser.mly"
                 ( false, false )
# 961 "src/GMLParser.ml"
               : 'state_type))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'state_open) in
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'state_type_init) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 486 "src/GMLParser.mly"
                 ( _2, false )
# 970 "src/GMLParser.ml"
               : 'state_type))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'state_open) in
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'state_type_final) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 490 "src/GMLParser.mly"
                 ( false, _2 )
# 979 "src/GMLParser.ml"
               : 'state_type))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'state_open) in
    let _2 = (Parsing.peek_val __caml_parser_env 2 : 'state_type_init) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'state_type_final) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 494 "src/GMLParser.mly"
                 ( _2, _3 )
# 989 "src/GMLParser.ml"
               : 'state_type))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'state_open) in
    let _2 = (Parsing.peek_val __caml_parser_env 2 : 'state_type_final) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'state_type_init) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 498 "src/GMLParser.mly"
                 ( _3, _2 )
# 999 "src/GMLParser.ml"
               : 'state_type))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'open_attribute) in
    Obj.repr(
# 502 "src/GMLParser.mly"
                               ()
# 1006 "src/GMLParser.ml"
               : 'state_open))
; (fun __caml_parser_env ->
    Obj.repr(
# 507 "src/GMLParser.mly"
           ( true )
# 1012 "src/GMLParser.ml"
               : 'state_type_init))
; (fun __caml_parser_env ->
    Obj.repr(
# 508 "src/GMLParser.mly"
          ( false )
# 1018 "src/GMLParser.ml"
               : 'state_type_init))
; (fun __caml_parser_env ->
    Obj.repr(
# 512 "src/GMLParser.mly"
         ( true )
# 1024 "src/GMLParser.ml"
               : 'state_type_final))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : 'open_attribute) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'bool_expr) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 524 "src/GMLParser.mly"
                 ( _4 )
# 1033 "src/GMLParser.ml"
               : 'state_invariant))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'open_attribute) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 528 "src/GMLParser.mly"
                 ( [] )
# 1041 "src/GMLParser.ml"
               : 'state_invariant))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'open_attribute) in
    Obj.repr(
# 531 "src/GMLParser.mly"
                                             ( [] )
# 1048 "src/GMLParser.ml"
               : 'state_invariant))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'transition) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'transitions) in
    Obj.repr(
# 539 "src/GMLParser.mly"
                          ( _1 :: _2 )
# 1056 "src/GMLParser.ml"
               : 'transitions))
; (fun __caml_parser_env ->
    Obj.repr(
# 540 "src/GMLParser.mly"
   ( [] )
# 1062 "src/GMLParser.ml"
               : 'transitions))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 14 : 'arc_id_opt) in
    let _3 = (Parsing.peek_val __caml_parser_env 13 : 'arc_type_opt) in
    let _6 = (Parsing.peek_val __caml_parser_env 10 : NumConst.t) in
    let _7 = (Parsing.peek_val __caml_parser_env 9 : 'arc_type_opt) in
    let _10 = (Parsing.peek_val __caml_parser_env 6 : NumConst.t) in
    let _11 = (Parsing.peek_val __caml_parser_env 5 : 'arc_id_opt) in
    let _12 = (Parsing.peek_val __caml_parser_env 4 : 'arc_type_opt) in
    let _14 = (Parsing.peek_val __caml_parser_env 2 : 'transition_body) in
    Obj.repr(
# 550 "src/GMLParser.mly"
                    ( _6, _10, _14 )
# 1076 "src/GMLParser.ml"
               : 'transition))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 14 : 'arc_id_opt) in
    let _3 = (Parsing.peek_val __caml_parser_env 13 : 'arc_type_opt) in
    let _6 = (Parsing.peek_val __caml_parser_env 10 : NumConst.t) in
    let _7 = (Parsing.peek_val __caml_parser_env 9 : 'arc_type_opt) in
    let _10 = (Parsing.peek_val __caml_parser_env 6 : NumConst.t) in
    let _11 = (Parsing.peek_val __caml_parser_env 5 : 'arc_id_opt) in
    let _12 = (Parsing.peek_val __caml_parser_env 4 : 'arc_type_opt) in
    let _14 = (Parsing.peek_val __caml_parser_env 2 : 'transition_body) in
    Obj.repr(
# 554 "src/GMLParser.mly"
                    ( _10, _6, _14 )
# 1090 "src/GMLParser.ml"
               : 'transition))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 0 : NumConst.t) in
    Obj.repr(
# 560 "src/GMLParser.mly"
                       ()
# 1097 "src/GMLParser.ml"
               : 'arc_id_opt))
; (fun __caml_parser_env ->
    Obj.repr(
# 561 "src/GMLParser.mly"
   ()
# 1103 "src/GMLParser.ml"
               : 'arc_id_opt))
; (fun __caml_parser_env ->
    Obj.repr(
# 565 "src/GMLParser.mly"
                                   ()
# 1109 "src/GMLParser.ml"
               : 'arc_type_opt))
; (fun __caml_parser_env ->
    Obj.repr(
# 566 "src/GMLParser.mly"
   ()
# 1115 "src/GMLParser.ml"
               : 'arc_type_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'updates) in
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'guard) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'label_name) in
    Obj.repr(
# 572 "src/GMLParser.mly"
                            ( _3, _2, _1 )
# 1124 "src/GMLParser.ml"
               : 'transition_body))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'updates) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'guard) in
    Obj.repr(
# 573 "src/GMLParser.mly"
                 ( NoSync, _2, _1 )
# 1132 "src/GMLParser.ml"
               : 'transition_body))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'updates) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'label_name) in
    Obj.repr(
# 574 "src/GMLParser.mly"
                      ( _2, [], _1 )
# 1140 "src/GMLParser.ml"
               : 'transition_body))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'label_name) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'guard) in
    Obj.repr(
# 575 "src/GMLParser.mly"
                    ( _1, _2, [] )
# 1148 "src/GMLParser.ml"
               : 'transition_body))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'label_name) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'updates) in
    Obj.repr(
# 576 "src/GMLParser.mly"
                      ( _1, [], _2 )
# 1156 "src/GMLParser.ml"
               : 'transition_body))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'guard) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'updates) in
    Obj.repr(
# 577 "src/GMLParser.mly"
                 ( NoSync, _1, _2 )
# 1164 "src/GMLParser.ml"
               : 'transition_body))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'label_name) in
    Obj.repr(
# 578 "src/GMLParser.mly"
              ( _1, [], [] )
# 1171 "src/GMLParser.ml"
               : 'transition_body))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'guard) in
    Obj.repr(
# 579 "src/GMLParser.mly"
         ( NoSync, _1, [] )
# 1178 "src/GMLParser.ml"
               : 'transition_body))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'updates) in
    Obj.repr(
# 580 "src/GMLParser.mly"
           ( NoSync, [], _1 )
# 1185 "src/GMLParser.ml"
               : 'transition_body))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : 'open_attribute) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'bool_expr) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 604 "src/GMLParser.mly"
                 ( _4 )
# 1194 "src/GMLParser.ml"
               : 'guard))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'open_attribute) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 607 "src/GMLParser.mly"
                  ( [] )
# 1202 "src/GMLParser.ml"
               : 'guard))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'open_attribute) in
    Obj.repr(
# 609 "src/GMLParser.mly"
                                         ( [] )
# 1209 "src/GMLParser.ml"
               : 'guard))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : 'open_attribute) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'update_list) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 622 "src/GMLParser.mly"
                 ( _4 )
# 1218 "src/GMLParser.ml"
               : 'updates))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'open_attribute) in
    Obj.repr(
# 625 "src/GMLParser.mly"
                                           ( [] )
# 1225 "src/GMLParser.ml"
               : 'updates))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'update) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'update_list) in
    Obj.repr(
# 632 "src/GMLParser.mly"
                      ( _1 :: _2 )
# 1233 "src/GMLParser.ml"
               : 'update_list))
; (fun __caml_parser_env ->
    Obj.repr(
# 633 "src/GMLParser.mly"
   ( [] )
# 1239 "src/GMLParser.ml"
               : 'update_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 5 : 'open_attribute) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : 'name) in
    let _5 = (Parsing.peek_val __caml_parser_env 1 : 'expression) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 642 "src/GMLParser.mly"
                 ( _4, _5 )
# 1249 "src/GMLParser.ml"
               : 'update))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : 'open_attribute) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'bool_expr_body) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 651 "src/GMLParser.mly"
                 ( _4 )
# 1258 "src/GMLParser.ml"
               : 'bool_expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'bool_expr_true) in
    Obj.repr(
# 656 "src/GMLParser.mly"
                  ( _1 )
# 1265 "src/GMLParser.ml"
               : 'bool_expr_body))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'conjunction) in
    Obj.repr(
# 657 "src/GMLParser.mly"
               ( _1 )
# 1272 "src/GMLParser.ml"
               : 'bool_expr_body))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'comparison) in
    Obj.repr(
# 658 "src/GMLParser.mly"
              ( [_1] )
# 1279 "src/GMLParser.ml"
               : 'bool_expr_body))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : 'open_attribute) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 664 "src/GMLParser.mly"
                 ( [] )
# 1287 "src/GMLParser.ml"
               : 'bool_expr_true))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 5 : 'open_attribute) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : 'bool_expr) in
    let _5 = (Parsing.peek_val __caml_parser_env 1 : 'bool_expr) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 672 "src/GMLParser.mly"
                 ( list_append _4 _5 )
# 1297 "src/GMLParser.ml"
               : 'conjunction))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 5 : 'open_attribute) in
    let _2 = (Parsing.peek_val __caml_parser_env 4 : 'op) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : 'expression) in
    let _5 = (Parsing.peek_val __caml_parser_env 1 : 'expression) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 692 "src/GMLParser.mly"
                 ( Linear_constraint (_4 , _2 , _5) )
# 1308 "src/GMLParser.ml"
               : 'comparison))
; (fun __caml_parser_env ->
    Obj.repr(
# 701 "src/GMLParser.mly"
           ( OP_L )
# 1314 "src/GMLParser.ml"
               : 'op))
; (fun __caml_parser_env ->
    Obj.repr(
# 702 "src/GMLParser.mly"
             ( OP_LEQ )
# 1320 "src/GMLParser.ml"
               : 'op))
; (fun __caml_parser_env ->
    Obj.repr(
# 703 "src/GMLParser.mly"
            ( OP_EQ )
# 1326 "src/GMLParser.ml"
               : 'op))
; (fun __caml_parser_env ->
    Obj.repr(
# 704 "src/GMLParser.mly"
             ( OP_GEQ )
# 1332 "src/GMLParser.ml"
               : 'op))
; (fun __caml_parser_env ->
    Obj.repr(
# 705 "src/GMLParser.mly"
           ( OP_G )
# 1338 "src/GMLParser.ml"
               : 'op))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : 'open_attribute) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'linear_expression) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 715 "src/GMLParser.mly"
                  ( _4 )
# 1347 "src/GMLParser.ml"
               : 'expression))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'linear_term) in
    Obj.repr(
# 720 "src/GMLParser.mly"
               ( Linear_term _1 )
# 1354 "src/GMLParser.ml"
               : 'linear_expression))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'const) in
    Obj.repr(
# 724 "src/GMLParser.mly"
         ( Constant _1 )
# 1361 "src/GMLParser.ml"
               : 'linear_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'var) in
    Obj.repr(
# 725 "src/GMLParser.mly"
       ( _1 )
# 1368 "src/GMLParser.ml"
               : 'linear_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'name) in
    Obj.repr(
# 729 "src/GMLParser.mly"
        ( Variable (NumConst.one, _1) )
# 1375 "src/GMLParser.ml"
               : 'var))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 5 : 'open_attribute) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : 'const) in
    let _5 = (Parsing.peek_val __caml_parser_env 1 : 'name) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 732 "src/GMLParser.mly"
                 ( Variable (_4, _5) )
# 1385 "src/GMLParser.ml"
               : 'var))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : 'open_attribute) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'rational) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 741 "src/GMLParser.mly"
  ( _4 )
# 1394 "src/GMLParser.ml"
               : 'const))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : 'open_attribute) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'state_name_string) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 752 "src/GMLParser.mly"
                  ( _4 )
# 1403 "src/GMLParser.ml"
               : 'state_name))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 756 "src/GMLParser.mly"
        ( _1 )
# 1410 "src/GMLParser.ml"
               : 'state_name_string))
; (fun __caml_parser_env ->
    Obj.repr(
# 757 "src/GMLParser.mly"
   ( (* Increment the counter for unnamed locations *)
		nb_unnamed_locations := !nb_unnamed_locations + 1;
		(* Name the state *)
		"noname" ^ (string_of_int !nb_unnamed_locations)
	)
# 1420 "src/GMLParser.ml"
               : 'state_name_string))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : 'open_attribute) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'label_name_string) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 767 "src/GMLParser.mly"
                  ( _4 )
# 1429 "src/GMLParser.ml"
               : 'label_name))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 771 "src/GMLParser.mly"
        ( Sync _1 )
# 1436 "src/GMLParser.ml"
               : 'label_name_string))
; (fun __caml_parser_env ->
    Obj.repr(
# 772 "src/GMLParser.mly"
   ( NoSync )
# 1442 "src/GMLParser.ml"
               : 'label_name_string))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : 'open_attribute) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : string) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'close_attribute) in
    Obj.repr(
# 780 "src/GMLParser.mly"
                  ( _4 )
# 1451 "src/GMLParser.ml"
               : 'name))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'integer) in
    Obj.repr(
# 802 "src/GMLParser.mly"
         ( _1 )
# 1458 "src/GMLParser.ml"
               : 'rational))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'float) in
    Obj.repr(
# 803 "src/GMLParser.mly"
         ( _1 )
# 1465 "src/GMLParser.ml"
               : 'rational))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'integer) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'pos_integer) in
    Obj.repr(
# 804 "src/GMLParser.mly"
                              ( (NumConst.div _1 _3) )
# 1473 "src/GMLParser.ml"
               : 'rational))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'pos_integer) in
    Obj.repr(
# 808 "src/GMLParser.mly"
             ( _1 )
# 1480 "src/GMLParser.ml"
               : 'integer))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'pos_integer) in
    Obj.repr(
# 809 "src/GMLParser.mly"
                        ( NumConst.neg _2 )
# 1487 "src/GMLParser.ml"
               : 'integer))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : NumConst.t) in
    Obj.repr(
# 813 "src/GMLParser.mly"
     ( _1 )
# 1494 "src/GMLParser.ml"
               : 'pos_integer))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'pos_float) in
    Obj.repr(
# 817 "src/GMLParser.mly"
            ( _1 )
# 1501 "src/GMLParser.ml"
               : 'float))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'pos_float) in
    Obj.repr(
# 818 "src/GMLParser.mly"
                      ( NumConst.neg _2 )
# 1508 "src/GMLParser.ml"
               : 'float))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 822 "src/GMLParser.mly"
        ( 
		let fstr = _1 in
		let point = String.index fstr '.' in
		(* get integer part *)
		let f = if point = 0 then ref NumConst.zero else (
			let istr = String.sub fstr 0 point in
		  ref (NumConst.numconst_of_int (int_of_string istr))
		) in		
		(* add decimal fraction part *)
		let numconst_of_char = function
			| '0' -> NumConst.zero
			| '1' -> NumConst.one
			| '2' -> NumConst.numconst_of_int 2
			| '3' -> NumConst.numconst_of_int 3
			| '4' -> NumConst.numconst_of_int 4
			| '5' -> NumConst.numconst_of_int 5
			| '6' -> NumConst.numconst_of_int 6
			| '7' -> NumConst.numconst_of_int 7
			| '8' -> NumConst.numconst_of_int 8
			| '9' -> NumConst.numconst_of_int 9
			| _ ->  raise (ParsingError (0,0)) in
		let ten = NumConst.numconst_of_int 10 in
		let dec = ref (NumConst.numconst_of_frac 1 10) in
		for i = point+1 to (String.length fstr) - 1 do
			let c = fstr.[i] in
			let d = numconst_of_char c in
			f := NumConst.add !f (NumConst.mul !dec d);
			dec := NumConst.div !dec ten 
		done;		
		!f
	)
# 1545 "src/GMLParser.ml"
               : 'pos_float))
(* Entry main *)
; (fun __caml_parser_env -> raise (Parsing.YYexit (Parsing.peek_val __caml_parser_env 0)))
|]
let yytables =
  { Parsing.actions=yyact;
    Parsing.transl_const=yytransl_const;
    Parsing.transl_block=yytransl_block;
    Parsing.lhs=yylhs;
    Parsing.len=yylen;
    Parsing.defred=yydefred;
    Parsing.dgoto=yydgoto;
    Parsing.sindex=yysindex;
    Parsing.rindex=yyrindex;
    Parsing.gindex=yygindex;
    Parsing.tablesize=yytablesize;
    Parsing.table=yytable;
    Parsing.check=yycheck;
    Parsing.error_function=parse_error;
    Parsing.names_const=yynames_const;
    Parsing.names_block=yynames_block }
let main (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (Parsing.yyparse yytables 1 lexfun lexbuf : ParsingStructure.parsing_structure)
